﻿using System;
using System.Collections.Generic;
using System.IO;
using Windows.Storage;
using Windows.UI.StartScreen;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace RandomView {
  public sealed partial class MainPage : Page {
    private const string DataFolderString = "\"DataFolder\"";
    private const string ErrorTitle = "エラー";
    private ApplicationDataContainer LocalSettings = ApplicationData.Current.LocalSettings;
    private StorageFolder LocalFolder = ApplicationData.Current.LocalFolder;
    private List<string> AllFile = new List<string>();
    private List<string> Candidate = new List<string>();
    private List<string> History = new List<string>();
    private System.Random Rnd = new System.Random();
    private System.Diagnostics.Stopwatch sw = new System.Diagnostics.Stopwatch();

    public MainPage() {
      this.InitializeComponent();
    }

    protected override async void OnNavigatedTo(NavigationEventArgs e) {
      ScrollViewerMain.HorizontalScrollBarVisibility = ScrollBarVisibility.Auto;
      ScrollViewerMain.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;
      SetZoomButton();

      var parameter = (string)e.Parameter;
      if (parameter != "")
        DataFolderPath = parameter;
      else
        if (LocalSettings.Values[DataFolderString] != null)
          DataFolderPath = LocalSettings.Values[DataFolderString].ToString();
    }

    private void SetZoomButton() {
      ButtonOriginalSize.IsEnabled = !(ScrollViewerMain.ZoomFactor > 0.999f &&
        ScrollViewerMain.ZoomFactor < 1.001f);
    }

    private string DataFolderPathValue = "";
    private string DataFolderPath {
      get { return DataFolderPathValue; }
      set {
        LoadList(value, false);
        DataFolderPathValue = value;
        LocalSettings.Values[DataFolderString] = value;
        TextBoxFolder.Text = value;
        ButtonAddTile.IsEnabled = value != "";
      }
    }

    private async void LoadList(string path, bool mustUpdate) {
      var listString = "";
      var listFileName = ConvertPath(path, true);
      StorageFile sFile;
      if (!mustUpdate)
        try {
          sFile = await LocalFolder.GetFileAsync(listFileName);
          listString = await FileIO.ReadTextAsync(sFile);
        } catch (Exception) {
          mustUpdate = true;
        }
      AllFile.Clear();
      if (mustUpdate) await GetFileList(path);
      else AllFile.AddRange(listString.Split(
        new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries));

      ClearHistory();
      ImageMain.Source = null;

      if (AllFile.Count > 0) {
        TextBlockInformation.Text = AllFile.Count + " files in " + path;
        ButtonRandom.IsEnabled = true;
      } else ButtonRandom.IsEnabled = false;
    }

    private string ConvertPath(string path, bool addExtention) {
      path = path.Replace(@":\", "__");
      path = path.Replace(@"\", "_");
      if (addExtention) path += ".txt";
      return path;
    }

    private async System.Threading.Tasks.Task GetFileList(string path) {
      await ShowMessage(path + "の探索を始めます");
      sw.Reset();
      sw.Start();

      StorageFolder sFolder;
      try {
        sFolder = await StorageFolder.GetFolderFromPathAsync(path);
      } catch (Exception ex) {
        ShowMessage("GetFileList : " + ex.Message, ErrorTitle);
        return;
      }

      var items = await sFolder.GetItemsAsync();
      string[] extensions = new string[] { ".jpg", ".jpeg", ".bmp" };

      foreach (var item in items) {
        foreach (var extension in extensions)
          if (System.IO.Path.GetExtension(item.Name).ToLower() == extension) {
            AllFile.Add(item.Name);
            break;
          }
      }
      items = null;

      StorageFile sFile = await LocalFolder.CreateFileAsync(ConvertPath(path, true),
        CreationCollisionOption.ReplaceExisting);
      await FileIO.WriteLinesAsync(sFile, AllFile);

      sw.Stop();
      await ShowMessage(path + "の探索終了（" + sw.ElapsedMilliseconds + "ミリ秒）");
    }

    private async System.Threading.Tasks.Task ShowMessage(string message) {
      var md = new Windows.UI.Popups.MessageDialog(message);
      await md.ShowAsync();
    }

    private async System.Threading.Tasks.Task ShowMessage(string message, string title) {
      var md = new Windows.UI.Popups.MessageDialog(message);
      md.Title = title;
      await md.ShowAsync();
    }

    private void ClearHistory() {
      History.Clear();
      Candidate.Clear();
      Candidate.AddRange(AllFile);
      HistoryIndex = 0;
    }

    private int HistoryIndexValue;
    private int HistoryIndex {
      get { return HistoryIndexValue; }
      set {
        HistoryIndexValue = value;
        if (History.Count < 1) {
          ButtonPrevious.IsEnabled = false;
          ButtonNext.IsEnabled = false;
        } else {
          ButtonPrevious.IsEnabled = History.Count - 1 + HistoryIndex > 0;
          ButtonNext.IsEnabled = HistoryIndex < 0;
        }
      }
    }

    private async void ButtonSetFolder_Click(object sender, RoutedEventArgs e) {
      var s = TextBoxFolder.Text;
      if (s == "") {
        await ShowMessage("フォルダー名が入力されていません", ErrorTitle);
        return;
      }

      StorageFolder sFolder;
      try {
        sFolder = await StorageFolder.GetFolderFromPathAsync(s);
      } catch (Exception ex) {
        ShowMessage("ButtonSetFolder_Click : " + ex.Message, ErrorTitle);
        return;
      }

      DataFolderPath = sFolder.Path;
    }

    private void ButtonUpdate_Click(object sender, RoutedEventArgs e) {
      LoadList(DataFolderPath, true);
    }

    private void ButtonRandom_Click(object sender, RoutedEventArgs e) {
      var index = Rnd.Next(Candidate.Count);
      var fileName = Candidate[index];
      LoadImage(System.IO.Path.Combine(DataFolderPath, fileName));
    }

    private async void LoadImage(string fileName) {
      StorageFile sFile;
      try {
        sFile = await StorageFile.GetFileFromPathAsync(fileName);
      } catch (Exception ex) {
        ShowMessage("LoadImage : " + ex.Message, ErrorTitle);
        return;
      }

      var stream = await sFile.OpenAsync(FileAccessMode.Read);
      var bi = new Windows.UI.Xaml.Media.Imaging.BitmapImage();
      bi.SetSource(stream);
      ImageMain.Source = bi;
      TextBlockInformation.Text = fileName;

      var sv = ScrollViewerMain;
      sv.ScrollToHorizontalOffset(0.0);
      sv.ScrollToVerticalOffset(0.0);
      sv.ZoomToFactor(1.0f);
    }

    private void ButtonPrevious_Click(object sender, RoutedEventArgs e) {
      HistoryIndex--;
      var fileName = History[History.Count - 1 + HistoryIndex];
      LoadImage(Path.Combine(new string[] { DataFolderPath, fileName }));
    }

    private void ButtonNext_Click(object sender, RoutedEventArgs e) {
      HistoryIndex++;
      var fileName = History[History.Count - 1 + HistoryIndex];
      LoadImage(Path.Combine(new string[] { DataFolderPath, fileName }));
    }

    private void ButtonOriginalSize_Click(object sender, RoutedEventArgs e) {
      ScrollViewerMain.ZoomToFactor(1.0f);
    }

    private void ScrollViewerMain_ViewChanged(object sender, ScrollViewerViewChangedEventArgs e) {
      SetZoomButton();
    }

    private void ScrollViewerMain_LayoutUpdated(object sender, object e) {
      ScrollViewerMain.MaxWidth = this.ActualWidth;
    }

    private async void ButtonAddTile_Click(object sender, RoutedEventArgs e) {
      if (DataFolderPath == "") return;
      var s = ConvertPath(DataFolderPath, false);

      var st = new SecondaryTile();
      st.TileId = s;
      st.ShortName = s;
      st.DisplayName = s;
      st.Arguments = DataFolderPath;
      st.TileOptions = TileOptions.ShowNameOnLogo;
      st.Logo = new Uri("ms-appx:///Assets/Logo.png");

      var isPinned = await st.RequestCreateAsync();
      if (isPinned) await ShowMessage("タイルを追加しました");
    }

    private void ButtonExit_Click(object sender, RoutedEventArgs e) {
      App.Current.Exit();
    }
  }
}
